"""
Utility module for hbllmutils package.

This module provides utility functions and classes for the hbllmutils package,
including logging utilities and other helper functions. It serves as a central
point for importing commonly used utility components.

The module exports:
    - get_global_logger: Function to access the global logger instance
    - log_pformat: Function to format and truncate complex data structures for logging

Example::
    >>> from hbllmutils.utils import get_global_logger, log_pformat
    >>> logger = get_global_logger()
    >>> logger.info("Starting application")
    >>> data = {"key": "value" * 1000}
    >>> logger.debug(log_pformat(data))
"""

from .logging import get_global_logger
from .truncate import log_pformat, truncate_dict
